// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file TranslateLsFct.h */

#ifndef AMDIS_TRANSLATELSFCT_H
#define AMDIS_TRANSLATELSFCT_H

#include "ElementFunction.h"
#include "FixVec.h"

using namespace AMDiS;

template<typename T>
class TranslateLsFct : public ElementFunction<T>
{
public:
  /// Constructor
  TranslateLsFct(ElementFunction<T> *f_, double c_)
    : ElementFunction<T>(),
      f(f_),
      c(c_)
  {}

  /** 
   * evaluation at given coordinates.
   *
   * Defines new level set function. Zero level set of this level set
   * function is level set c of level set function f.
   */
  const T& operator()(const DimVec<double>& bary) const 
  {
    f->setElInfo(this->elInfo);
    return ((*f)(bary) - c);
  };

protected:
  /// Level set function which is translated.
  ElementFunction<T> *f;

  /// Translation parameter. 
  double c;
};

#endif  // AMDIS_TRANSLATELSFCT_H
