#include "parallel/Mtl4Solver.h"
#include "parallel/PITL_Solver.h"
#include "parallel/ParallelMapper.h"

namespace AMDiS {
  namespace Parallel {

    void Mtl4Solver::addPMTLSolvers()
    {
      OEMSolverCreator *creator;

      creator = new CGSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pcg", creator);

      creator = new CGSSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pcgs", creator);

      creator = new BiCGSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pbicg", creator);

      creator = new BiCGStabSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pbicgstab", creator);

      creator = new BiCGStab2Solver::Creator;
      CreatorMap< OEMSolver >::addCreator("pbicgstab2", creator);

      creator = new BiCGStabEllSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pbicgstab_ell", creator);

      creator = new QMRSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pqmr", creator);

      creator = new TFQMRSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("ptfqmr", creator);

      creator = new GMResSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pgmres", creator);

      creator = new IDRsSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pidr_s", creator);

      creator = new MinResSolver::Creator;
      CreatorMap< OEMSolver >::addCreator("pminres", creator);
    }

/*    void Mtl4Solver::addPMTLPrecons() 
    {
       ParallelPreconditionCreator *creator;

       creator =  new DiagonalPreconditioner::Creator;
       CreatorMap<ParallelPreconditioner >::addCreator("pdiag", creator);

       creator = new ILUPreconditioner::Creator;
       CreatorMap<ParallelPreconditioner >::addCreator("pilu", creator);

       creator = new ICPreconditioner::Creator;
       CreatorMap<ParallelPreconditioner >::addCreator("pic", creator);

       creator =  new IdentityPreconditioner::Creator;
       CreatorMap<ParallelPreconditioner >::addCreator("pno", creator);

    }*/
  
    void Mtl4Solver::createSolver()
    {
      FUNCNAME("Parallel::Mtl4Solver::createSolver()");

      // === create solver ===
      std::string solverType("0");
      Parameters::get(name + "->solver", solverType);
      solverType = "p" + solverType;
      OEMSolverCreator *solverCreator = 
	dynamic_cast<OEMSolverCreator*>(CreatorMap<OEMSolver>::getCreator(solverType));
      TEST_EXIT(solverCreator)("no solver type\n");
      solverCreator->setName(name + "->solver");
      solver = solverCreator->create();
      solver->initParameters();
    }

    //TODO: replace the name in the map
    void Mtl4Solver::createPrecon()
    {
      /*std::string preconType("no");
      GET_PARAMETER(0, name + "->solver->left precon", &preconType);
      preconType = "p" + preconType ;

      CreatorInterface<BasePreconditioner> *preconCreator = 
	CreatorMap<BasePreconditioner>::getCreator(preconType);

  //    solver->setLeftPrecon(preconCreator->create(solverMatrix.getMatrix()));
      solver->setLeftPrecon(preconCreator);

      preconType= "no";
      GET_PARAMETER(0, name + "->solver->right precon", &preconType);
      preconType = "p" + preconType;

      preconCreator = CreatorMap<BasePreconditioner>::getCreator(preconType);
  //    solver->setRightPrecon(preconCreator->create(solverMatrix.getMatrix()));
      solver->setRightPrecon(preconCreator);*/
    }
    
    void Mtl4Solver::solve(AdaptInfo* adaptInfo, bool fixedMatrix)
    {
/*      FUNCNAME("Mtl4Solver::solve()");
      std::string solverName("");
      GET_PARAMETER(0, name+"->solver", &solverName);
      TEST_EXIT(solverName != "")("need solver name");
      OEMSolver* solver = getPMTLSolver(solverName);

      delete solver;
      */
      ParallelMapper pmapper(*meshDistributor, nComponents);
      solver->solveSystem(solverMatrix, *solution, *rhs, pmapper);
    }
  }

  template< > 
  void CreatorMap< Parallel::ParallelPreconditioner >::addDefaultCreators() {
    Parallel::ParallelPreconditionCreator *creator;

     creator =  new Parallel::DiagonalPreconditioner::Creator;
     addCreator("diag", creator);

     creator = new Parallel::ILUPreconditioner::Creator;
     addCreator("ilu", creator);

     creator = new Parallel::ICPreconditioner::Creator;
     addCreator("ic", creator);

     creator =  new Parallel::IdentityPreconditioner::Creator;
     addCreator("no", creator);
  }
}
